package net.sf.sql2java.fetching.sql.filter;

import net.sf.sql2java.common.beans.BreadCrumbs;
import net.sf.sql2java.common.beans.SourceFilter;
import net.sf.sql2java.common.beans.BreadCrumbs.BreadCrumb;
import net.sf.sql2java.fetching.sql.SQLFetcher.Section;

import org.apache.log4j.Logger;

public class RegexSourceFilter 
	extends SourceFilter 
{
	private Logger LOGGER = Logger.getLogger(RegexSourceFilter.class);
	public enum Props { 
		configFile, 
		filterMode, 
		databaseExpression, 
		schemaExpression, 
		tableExpression, 
		columnExpression 
	}
	
	private static final String DEFAULT_FILTER_MODE = "inclusive";	
	
// -- Constructors ------------------------------------------------------------
	
	public RegexSourceFilter() { }
	
// -- Matching Methods --------------------------------------------------------

	@Override
	public boolean match(BreadCrumbs breadcrumbs) {
		boolean internalResult = isAMatch(breadcrumbs);
		
		return (isInclusiveFilter()) ? internalResult : (! internalResult);
	}
	
	protected boolean isAMatch(BreadCrumbs breadcrumbs) {
		// -- make a copy of the provided crumbs so we won't disturb it's
		// -- internal structure
		BreadCrumbs crumbs = new BreadCrumbs();
		crumbs.addAll(breadcrumbs);
		
		// -- ok, lets start with our match-making
		BreadCrumb crumb;
		while (! crumbs.isEmpty()) {
			// -- pop the current crumb
			crumb = crumbs.pop();
			
			// -- check if the crumb matches
			boolean isMatch = isAMatch(crumb);
			
			// -- if it didn't match, return false since only one failure is
			// -- sufficiant to fail the whole matching process
			if (! isMatch) 
				return false;
		}
		
		// -- if we got here, no matching of crumbs failed. In this
		// -- case we can conclude the crumbs match
		return true;
	}
	
	protected boolean isAMatch(BreadCrumb crumb) {
		boolean result = false;
		
		switch (Section.valueOf(crumb.getSection())) {
		case database:
			result = crumb.getChild().matches(getDatabaseExpression());
			break;
		case schema:
			result = crumb.getChild().matches(getSchemaExpression());
			break;
		case table:
			result = crumb.getChild().matches(getTableExpression());
			break;
		case column:
			result = crumb.getChild().matches(getColumnExpression());
			break;
		default:
			LOGGER.warn(
					String.format("Unable to match unknown section '%s'", crumb.getSection())
			);
			result = false;
		}
		
		return result;
	}
	
// -- Getters -----------------------------------------------------------------
	
	public boolean isInclusiveFilter() {
		return (getProperty(Props.filterMode.toString(), DEFAULT_FILTER_MODE).equalsIgnoreCase("inclusive"));
	}
	
	public boolean isExclusiveFilter() {
		return (getProperty(Props.filterMode.toString(), DEFAULT_FILTER_MODE).equalsIgnoreCase("exclusive"));
	}
	
	public String getColumnExpression() {
		return getProperty(Section.column.toString(), ".*");
	}
	
	public String getDatabaseExpression() {
		return getProperty(Section.database.toString(), ".*");
	}
	
	public String getSchemaExpression() {
		return getProperty(Section.schema.toString(), ".*");
	}
	
	public String getTableExpression() {
		return getProperty(Section.table.toString(), ".*");
	}
}
